uniform sampler2D Texture_Diffuse;
uniform sampler2D Texture_Spec_Mask;
uniform sampler2D Texture_EnvMap;

uniform lowp vec3 U_Recolour;
uniform lowp vec3 U_Light1Col;
uniform lowp vec3 U_Light2Col;
uniform lowp vec3 U_Light1Dir; // world space
uniform lowp vec3 U_Light2Dir; // world space
uniform lowp vec3 U_SkyCol;
uniform lowp vec3 U_GroundCol;
uniform lowp vec3 U_FogCol;
uniform lowp float U_SkyVis;

varying mediump vec3 PS_ReflectDir;
varying lowp vec3 PS_DiffuseCol;
varying mediump vec2 PS_TexCoord;
varying mediump vec2 PS_EnvTexCoord;
varying mediump float PS_Fresnel;
varying lowp float PS_FogFactor;
varying lowp float PS_SkyVis;

void main(void)
{
    mediump vec3 normReflectDir = normalize(PS_ReflectDir);
    
    lowp vec3 diffuse = texture2D(Texture_Diffuse, PS_TexCoord).rgb*0.3;
    lowp float mask = texture2D(Texture_Spec_Mask, PS_TexCoord).r;
    lowp vec4 reflectionFull = texture2D(Texture_EnvMap, PS_EnvTexCoord);
    mediump float specularity = texture2D(Texture_Spec_Mask, PS_TexCoord).g;
    diffuse += mask*U_Recolour;

    mediump float specExponent = 50.0;
    mediump float specNormalise = 2.32*specularity;
    
    mediump float spec1 = max(dot(normReflectDir, U_Light1Dir), 0.0);
    spec1 = pow(spec1, specExponent)*specNormalise;
    mediump float spec2 = max(dot(normReflectDir, U_Light2Dir), 0.0);
    spec2 = pow(spec2, specExponent)*specNormalise;
    
    lowp float reflectionGrey = reflectionFull.a;
    lowp vec3 reflection = mix((1.999*reflectionGrey)*U_GroundCol, reflectionFull.rgb, PS_SkyVis);
    
    lowp vec3 diffuse_col = diffuse*PS_DiffuseCol;
    lowp vec3 specular_col = U_Light1Col*spec1 + U_Light2Col*spec2 + reflection*specularity;
    lowp vec3 colour = mix(diffuse_col, specular_col, PS_Fresnel);
    
    gl_FragColor.rgb = mix(colour, U_FogCol, PS_FogFactor);
    
    /*
    //highp vec3 normal = normalize(PS_Normal);
    //highp vec3 normal = vec3(0.0,0.0,1.0);
    highp vec3 normal = texture2D(Texture_Normal, PS_TexCoord).rgb;
    normal.x = normal.x*2.0 - 1.0;
    normal.y = normal.y*2.0 - 1.0;
    normal.x *= abs(normal.x);
    normal.y *= abs(normal.y);
    //normal.z = sqrt(1.0 - normal.x*normal.x - normal.y*normal.y);
    normal = normalize(PS_TangentToWorldSpace*normal);
    //lowp vec3 normal = texture2D(Texture_Normal, PS_TexCoord).rgb;
    lowp vec3 normCamDir = normalize(PS_CameraDir);
    //normal = normalize(normal*1.999 - 1.0); // currently unused
    
    mediump vec2 diffuseEnvCoords = 0.331 + 0.0244 + normal.xy*0.0244;
    lowp vec3 diffuseReflection = texture2D(Texture_EnvMap, diffuseEnvCoords).rgb;
    
    lowp vec3 diffuse = texture2D(Texture_Diffuse, PS_TexCoord).rgb;
    lowp vec2 spec_mask = texture2D(Texture_Spec_Mask, PS_TexCoord).rg;
    diffuse += spec_mask.r*U_Recolour;
    //diffuse = mix(diffuse, diffuse*U_Recolour, spec_mask.r);
    
    //mediump float specExponent = spec_mask.g;
    //specExponent = 1.0 + 200.0*specExponent*specExponent;
    //mediump float specNormalise = (specExponent + 8.0)*0.04; //*0.5; //*0.35
    
    lowp float fresnel = dot(normCamDir, normal);
    lowp vec3 reflectDir = normCamDir - 1.999*fresnel*normal;
    
    //mediump vec2 texCoord = 0.331 + 0.1655 + reflectDir.xy*0.1655;
    mediump vec2 texCoord = 0.331 + 0.1655 + reflectDir.xy*0.15;
    lowp vec4 reflectionFull = texture2D(Texture_EnvMap, texCoord);
    
    mediump float specularity = spec_mask.g;
    mediump float specExponent = 150.0;
    mediump float specNormalise = 6.32*specularity; // (specExponent + 8.0)*0.04*specularity;
    
    // lambert
    lowp float diff1 = max(dot(normal, U_Light1Dir), 0.0);
    lowp float diff2 = max(dot(normal, U_Light2Dir), 0.0);
    
    // phong
    lowp float spec1 = max(dot(reflectDir, U_Light1Dir), 0.0);
    spec1 = pow(spec1, specExponent)*specNormalise;
    lowp float spec2 = max(dot(reflectDir, U_Light2Dir), 0.0);
    spec2 = pow(spec2, specExponent)*specNormalise;
    
    //lowp vec3 colour = ((U_AmbientCol + diff1*U_Light1Col + diff2*U_Light2Col)*diffuse) + (spec1*U_Light1Col + spec2*U_Light2Col);

    fresnel = clamp(1.0 + fresnel, 0.0, 1.0);
    //fresnel = 1.0 + fresnel;
    fresnel *= fresnel;
    fresnel = 0.2 + (0.8*fresnel*fresnel);
    
    lowp float diffuseVis = smoothstep(U_SkyVis-0.3, U_SkyVis+0.3, normal.y);
    //lowp vec3 diffuseReflection = mix(vec3(0.06,0.06,0.06), U_SkyCol, diffuseVis);
    diffuseReflection = mix(vec3(0.06,0.06,0.06), diffuseReflection, diffuseVis);
    
    // reflectionGrey scaled by U_AmbientCol+U_Light1Col+U_Light2Col to approximate variations in nearby lighting
    lowp float reflectionGrey = reflectionFull.a*1.999; // should ideally scale up in original texture
    lowp float skyVis = smoothstep(U_SkyVis-0.05, U_SkyVis+0.05, reflectDir.y);
    lowp vec3 reflection = mix(vec3(reflectionGrey, reflectionGrey, reflectionGrey)*(U_AmbientCol+U_Light1Col+U_Light2Col), reflectionFull.rgb, skyVis);
    
    lowp vec3 diffuse_col = (U_AmbientCol + diff1*U_Light1Col + diff2*U_Light2Col + diffuseReflection)*diffuse;
    lowp vec3 specular_col = spec1*U_Light1Col + spec2*U_Light2Col + reflection*specularity;
    lowp vec3 colour = mix(diffuse_col, specular_col, fresnel);
    
    //colour = reflectDir*0.5 + 0.5;
    //colour = reflection;
    //colour = diffuseReflection;
    //colour = U_AmbientCol;
    
    //lowp vec3 colour = normal;
    gl_FragColor.rgb = mix(colour, vec3(0.22,0.22,0.28), PS_FogFactor);
     */
}
