uniform sampler2D Texture_Diffuse;
uniform sampler2D Texture_Spec_Mask;
uniform sampler2D Texture_EnvMap;

uniform lowp vec3 U_Recolour;
uniform lowp vec3 U_Light1Col;
uniform lowp vec3 U_Light2Col;
uniform lowp vec3 U_Light1Dir; // world space
uniform lowp vec3 U_Light2Dir; // world space
uniform lowp vec3 U_SkyCol;
uniform lowp vec3 U_GroundCol;
uniform lowp vec3 U_FogCol;
uniform lowp float U_SkyVis;

varying mediump vec3 PS_ReflectDir;
varying lowp vec3 PS_DiffuseCol;
varying mediump vec2 PS_TexCoord;
varying mediump vec2 PS_EnvTexCoord;
varying mediump float PS_Fresnel;
varying lowp float PS_FogFactor;
varying lowp float PS_SkyVis;

void main(void)
{
    mediump vec3 normReflectDir = normalize(PS_ReflectDir);
    
    mediump float specularity = texture2D(Texture_Spec_Mask, PS_TexCoord).g;
    lowp vec3 diffuse = texture2D(Texture_Diffuse, PS_TexCoord).rgb;
    lowp float mask = texture2D(Texture_Spec_Mask, PS_TexCoord).r;
    mediump float texBias = (1.0-specularity)*5.0;
    lowp vec4 reflectionFull = texture2D(Texture_EnvMap, PS_EnvTexCoord, texBias);
    //diffuse += mask*U_Recolour;
    diffuse = max(diffuse, mask*U_Recolour);

    // this is different to hq version - change?
    mediump float specExponent = 150.0;
    mediump float specNormalise = 6.32*specularity;
    
    mediump float spec1 = max(dot(normReflectDir, U_Light1Dir), 0.0);
    spec1 = pow(spec1, specExponent)*specNormalise;
    mediump float spec2 = max(dot(normReflectDir, U_Light2Dir), 0.0);
    spec2 = pow(spec2, specExponent)*specNormalise;
    
    lowp float reflectionGrey = reflectionFull.a;
    lowp vec3 reflection = mix((1.999*reflectionGrey)*U_GroundCol, reflectionFull.rgb, PS_SkyVis);
    
    lowp vec3 diffuse_col = diffuse*PS_DiffuseCol;
    lowp vec3 specular_col = U_Light1Col*spec1 + U_Light2Col*spec2 + reflection;
    lowp vec3 colour = mix(diffuse_col, specular_col, PS_Fresnel);
    
    gl_FragColor.rgb = mix(colour, U_FogCol, PS_FogFactor);
}
