//uniform sampler2D Texture4; // env map

uniform mat4 PTransform; // camera->screen space
uniform mat4 MVTransform; // object->camera space
uniform mat4 MTransform; // object->world space
uniform vec3 U_CameraPos; // in world space, but maybe different scale?

uniform vec3 U_Recolour;
uniform vec3 U_AmbientCol;
uniform vec3 U_Light1Col;
uniform vec3 U_Light2Col;
uniform vec3 U_Light1Dir; // world space
uniform vec3 U_Light2Dir; // world space
uniform vec3 U_SkyCol;
uniform vec3 U_GroundCol;
uniform float U_SkyVis;
uniform float U_FogDist;

// in object space
attribute vec4 VS_Position;
attribute vec3 VS_Normal;
attribute vec2 VS_TexCoord;

varying vec2 PS_TexCoord;
varying vec3 PS_DiffuseCol;
varying float PS_FogFactor;
 
void main(void)
{
    float dist = length((MVTransform*VS_Position).xyz);
    
    gl_Position = PTransform * MVTransform * VS_Position;
    
    vec3 worldNormal = normalize(mat3(MTransform)*VS_Normal);
    
    lowp float adjSkyVis = U_SkyVis*1.4;
    lowp float diffuseVis = clamp(smoothstep(adjSkyVis-0.7, adjSkyVis+0.2, worldNormal.y), 0.0, 1.0);
    lowp vec3 diffuseReflection = mix(U_GroundCol, U_SkyCol, diffuseVis);
    
    // "correct"
    //float diff1 = max(dot(worldNormal, U_Light1Dir), 0.0);
    //float diff2 = max(dot(worldNormal, U_Light2Dir), 0.0);
    
    // approx subsurface scattering
    float diff1 = dot(worldNormal, U_Light1Dir);
    diff1 = max(diff1, 0.0) - 0.3*min(diff1, 0.0);
    float diff2 = dot(worldNormal, U_Light2Dir);
    diff2 = max(diff2, 0.0) - 0.3*min(diff2, 0.0);
    
    PS_DiffuseCol = U_AmbientCol + U_Light1Col*diff1 + U_Light2Col*diff2 + diffuseReflection;
    
    PS_TexCoord = VS_TexCoord*2.0;
    
    PS_FogFactor = clamp(dist/U_FogDist, 0.0, 1.0); // was /9000.0
}
