uniform sampler2D Texture1; // alpha and diffuse map
uniform sampler2D Texture2; // specular and normal map
uniform sampler2D Texture4; // env map

uniform lowp vec3 U_SkyCol;
uniform lowp vec3 U_FogCol;

varying mediump vec4 PS_CameraDir;
varying lowp vec4 PS_Light1Dir;
varying lowp vec3 PS_Light1Col;
varying lowp vec3 PS_Light2Dir;
varying lowp vec3 PS_Light2Col;
varying lowp vec3 PS_RemainingCol;
varying lowp vec3 PS_SkyDir;
//varying lowp vec2 PS_EnvMapOffset;
varying mediump vec2 PS_TexCoord0;
varying lowp float PS_FogFactor;

// todo: less normalises, as will only ever be rendered in distance

void main(void)
{
    //lowp vec4 textureCol = texture2D(Texture1, PS_TexCoord0, -0.5);
    lowp vec4 textureCol = texture2D(Texture1, PS_TexCoord0);
    
    if (textureCol.a < 0.4)
        discard;
    
    //lowp vec4 specNormCol = texture2D(Texture2, PS_TexCoord0, -0.5);
    lowp vec4 specNormCol = texture2D(Texture2, PS_TexCoord0);
    
    lowp vec3 normCamDir = normalize(PS_CameraDir.xyz);
    lowp vec3 normal = normalize(specNormCol.rgb*1.999 - 1.0); // renormalise is necessary for decent specular
    
    lowp float fresnel = dot(normCamDir, normal);
    lowp vec3 reflectDir = normCamDir - 1.999*fresnel*normal;
    
    //mediump vec2 envCoords = PS_EnvMapOffset + 0.1411 + reflectDir.xy*0.1655;
    //mediump vec2 envCoords = 0.1655 + reflectDir.xy*0.1655;
    mediump vec2 envCoords = vec2(PS_CameraDir.w,PS_Light1Dir.a) + reflectDir.xy*0.1655;
    lowp vec4 reflectionFull = texture2D(Texture4, envCoords);
    
    lowp vec3 normLight1Dir = normalize(PS_Light1Dir.xyz);
    lowp vec3 normLight2Dir = normalize(PS_Light2Dir);
    
    mediump float specExponent = specNormCol.a;
    //specExponent = 1.0 + 200.0*specExponent*specExponent;
    specExponent = 1.0 + 150.0*specExponent;
    mediump float specNormalise = (specExponent + 8.0)*0.04;
    
    lowp float spec1 = max(dot(reflectDir, normLight1Dir), 0.0);
    spec1 = pow(spec1, specExponent);
    lowp float spec2 = max(dot(reflectDir, normLight2Dir), 0.0);
    spec2 = pow(spec2, specExponent);
    
    lowp float diff1 = max(dot(normal, normLight1Dir), 0.0);
    lowp float diff2 = max(dot(normal, normLight2Dir), 0.0);
    lowp vec3 diffuse1 = diff1*PS_Light1Col;
    lowp vec3 diffuse2 = diff2*PS_Light2Col;
    
    lowp vec3 specular1 = (spec1*specNormalise)*PS_Light1Col;
    lowp vec3 specular2 = (spec2*specNormalise)*PS_Light2Col;
    
    lowp float reflectionGrey = reflectionFull.a*0.2;
    mediump float skyReflectVis = dot(PS_SkyDir, reflectDir);
    skyReflectVis = clamp((skyReflectVis*40.0)-1.5, 0.0, 1.0); // was ((skyReflectVis-0.05)*40.0)+0.5
    lowp vec3 reflection = mix(vec3(reflectionGrey, reflectionGrey, reflectionGrey), reflectionFull.rgb, skyReflectVis);
    
    lowp vec3 spec = specular1+specular2+reflection*specNormCol.a;
    
    mediump float skyDiffuseVis = dot(PS_SkyDir, normal);
    skyDiffuseVis = clamp((skyDiffuseVis+0.11)*2.5, 0.0, 1.0);
    lowp vec3 diffuseReflection = U_SkyCol*skyDiffuseVis;
    
    lowp vec3 colour = (PS_RemainingCol+diffuse1+diffuse2+diffuseReflection)*textureCol.rgb + spec;
    
    gl_FragColor.rgb = mix(colour, U_FogCol, PS_FogFactor);
}
