uniform mat4 PTransform; // camera->screen space
uniform mat4 MVTransform; // object->camera space
uniform vec3 U_CameraPos; // in world space
uniform float U_FogDist;

attribute vec4 VS_Position;
attribute vec4 VS_Normal;
attribute vec4 VS_Tangent;
attribute vec3 VS_Bitan;
attribute vec3 VS_Light1Dir;
attribute vec3 VS_Light2Dir;
attribute vec4 VS_Light1Col;
attribute vec4 VS_Light2Col;
attribute vec4 VS_RemainingCol;
attribute vec2 VS_TexCoord0;

varying vec4 PS_CameraDir;
varying vec4 PS_Light1Dir;
varying vec3 PS_Light1Col;
varying vec3 PS_Light2Dir;
varying vec3 PS_Light2Col;
varying vec3 PS_RemainingCol;
varying vec3 PS_SkyDir;
//varying vec2 PS_EnvMapOffset; // now in PS_CameraDir.w and PS_Light1Dir.w
varying vec2 PS_TexCoord0;
varying float PS_FogFactor;
 
void main(void)
{
    vec4 pos = MVTransform * VS_Position;
    gl_Position = PTransform*pos;
    float dist = length(pos.xyz);
    
    vec3 camDirInWorldSpace = (VS_Position.xyz-U_CameraPos)/50.0;
    
    mat3 tsprotation = mat3(VS_Tangent.x, VS_Bitan.x, VS_Normal.x,
                            VS_Tangent.y, VS_Bitan.y, VS_Normal.y,
                            VS_Tangent.z, VS_Bitan.z, VS_Normal.z);
    
    PS_CameraDir.xyz = tsprotation*camDirInWorldSpace;
    
    PS_SkyDir = vec3(VS_Tangent.y, VS_Bitan.y, VS_Normal.y);
    
    PS_CameraDir.w = VS_Normal.w;
    
    PS_FogFactor = clamp(dist/U_FogDist, 0.0, 1.0); // was /9000.0
    
    PS_Light1Dir.xyz = VS_Light1Dir;
    PS_Light1Dir.w = VS_Tangent.w;
    PS_Light1Col = VS_Light1Col.rgb;
    
    PS_Light2Dir = VS_Light2Dir;
    PS_Light2Col = VS_Light2Col.rgb;
    
    PS_RemainingCol = VS_RemainingCol.rgb - 0.25;
    
    PS_TexCoord0 = VS_TexCoord0*2.0;
}
