uniform sampler2D Texture0; // light remaining map
uniform sampler2D Texture1; // alpha and diffuse map
uniform sampler2D Texture2; // specular and normal map
uniform sampler2D Texture3; // light1/2 visibility - luminance+alpha
uniform sampler2D Texture4; // env map
uniform sampler2D Texture5; // env visibility

uniform lowp vec3 U_SkyCol;
uniform lowp vec3 U_FogCol;

varying lowp vec4 PS_Colour; // fog factor was separate, but caused problems on n7
varying lowp vec4 PS_EnvVisibility; // was last, but caused problems on n7
varying mediump vec3 PS_CameraDir;
varying lowp vec3 PS_Light1Dir;
varying lowp vec3 PS_Light1Col;
varying lowp vec3 PS_Light2Dir;
varying lowp vec3 PS_Light2Col;
varying lowp vec2 PS_EnvMapOffset;
varying mediump vec2 PS_TexCoord0;
varying mediump vec2 PS_TexCoord1;

void main(void)
{
    //lowp vec4 specNormCol = texture2D(Texture2, PS_TexCoord1, -0.5);
    lowp vec4 specNormCol = texture2D(Texture2, PS_TexCoord1);
    
    lowp vec3 normCamDir = normalize(PS_CameraDir);
    lowp vec3 normal = normalize(specNormCol.rgb*1.999 - 1.0); // renormalise is necessary for decent specular
    //lowp vec3 normal = vec3(0.0, 0.0, 1.0);
    
    mediump vec2 skyVisMapBase = (floor(PS_TexCoord0*64.0)+0.5)/64.0; // could avoid this by passing in more params, but this is slower
    
    lowp float fresnel = dot(normCamDir, normal);
    lowp vec3 reflectDir = normCamDir - 1.999*fresnel*normal;
    
    highp vec2 skyVisMapOffset = reflectDir.xy;
    
    //lowp vec4 skyVisMap = texture2D(Texture5, skyVisMapBase + skyVisMapOffset/128.0);
    lowp vec4 skyVisMap = texture2D(Texture5, skyVisMapBase + (skyVisMapOffset/136.5)); // to avoid bilerp bleeding from nearby texture in atlas
    
    highp vec2 diffuseVisMapOffset = normal.xy;
    
    //lowp vec4 diffuseVisMap = texture2D(Texture5, skyVisMapBase + diffuseVisMapOffset/128.0);
        lowp vec4 diffuseVisMap = texture2D(Texture5, skyVisMapBase + (diffuseVisMapOffset/136.5)); // to avoid bilerp bleeding from nearby texture in atlas
    
    mediump vec2 envCoords = PS_EnvMapOffset + 0.1411 + reflectDir.xy*0.1655;
    lowp vec4 reflectionFull = texture2D(Texture4, envCoords);
    
    //mediump vec2 diffuseEnvCoords = PS_EnvMapOffset + normal.xy*0.0244;
    //lowp vec3 diffuseReflection = texture2D(Texture4, diffuseEnvCoords).rgb;
    
    lowp vec3 normLight1Dir = normalize(PS_Light1Dir);
    lowp vec3 normLight2Dir = normalize(PS_Light2Dir);
    
    lowp float specularity = specNormCol.a;
    
    //fresnel = clamp(1.0 + fresnel, 0.0, 1.0);
    //fresnel *= fresnel;
    //fresnel *= fresnel;
    //specularity = specularity + ((1.0-specularity)*fresnel*fresnel);
    
    // energy conserving specular
    mediump float specExponent = specularity;
    //specExponent = 1.0 + 200.0*specExponent*specExponent;
    specExponent = 1.0 + 150.0*specExponent;
    mediump float specNormalise = (specExponent + 8.0)*0.04; //*0.5; //*0.35
    
    lowp vec3 remainingCol = texture2D(Texture0, PS_TexCoord0).rgb;
    lowp vec2 lightVis = texture2D(Texture3, PS_TexCoord0).ar;
    //lowp vec4 textureCol = texture2D(Texture1, PS_TexCoord1, -0.5);
    lowp vec4 textureCol = texture2D(Texture1, PS_TexCoord1);
    
    lowp float light1Vis = lightVis.y;
    lowp float light2Vis = lightVis.x;
    
    // phong
    lowp float spec1 = max(dot(reflectDir, normLight1Dir), 0.0);
    spec1 = pow(spec1, specExponent)*specNormalise;
    lowp float spec2 = max(dot(reflectDir, normLight2Dir), 0.0);
    spec2 = pow(spec2, specExponent)*specNormalise;
    lowp vec3 specular1 = (spec1*light1Vis)*PS_Light1Col;
    lowp vec3 specular2 = (spec2*light2Vis)*PS_Light2Col;
    
    //remainingCol -= 0.25;
    
    mediump float skyVis = dot(skyVisMap, PS_EnvVisibility);
    
    lowp float diffuseVis = dot(diffuseVisMap, PS_EnvVisibility);
    
    lowp float reflectionGrey = reflectionFull.a*0.2; //*0.33;
    
    skyVis = clamp((skyVis-0.4)*5.0, 0.0, 1.0);
    
    lowp vec3 reflection = mix(vec3(reflectionGrey, reflectionGrey, reflectionGrey), reflectionFull.rgb, skyVis);
    
    // lambert
    lowp float diff1 = max(dot(normal, normLight1Dir), 0.0);
    lowp float diff2 = max(dot(normal, normLight2Dir), 0.0);
    lowp vec3 diffuse1 = (diff1*light1Vis)*PS_Light1Col;
    lowp vec3 diffuse2 = (diff2*light2Vis)*PS_Light2Col;
    
    lowp vec3 spec = (specular1+specular2+reflection*specularity)/textureCol.a; // was specNormCol.a*0.6
    
    //diffuseReflection = mix(vec3(0.06,0.06,0.06), diffuseReflection, diffuseVis);
    lowp vec3 diffuseReflection = mix(vec3(0.06,0.06,0.06), U_SkyCol, diffuseVis);
    
    lowp vec3 texCol = textureCol.rgb;
    
    gl_FragColor.a = textureCol.a;
    
    //texCol *= texCol;
    
    
    // not gamma correct
    //lowp vec3 colour = (PS_Colour.rgb+diffuse1+diffuse2+diffuseReflection+remainingCol)*texCol*(1.0-specularity) + spec;
    lowp vec3 colour = (PS_Colour.rgb+diffuse1+diffuse2+diffuseReflection+remainingCol)*texCol + spec;
    
    //colour = PS_Colour.rgb+diffuse1+diffuse2+diffuseReflection+remainingCol;
    
    //colour.r = pow(colour.r, 1.0/1.5);
    //colour.g = pow(colour.g, 1.0/1.5);
    //colour.b = pow(colour.b, 1.0/1.5);
    
    gl_FragColor.rgb = mix(colour, U_FogCol, PS_Colour.a);
}
