uniform mat4 PTransform;
uniform mat4 MVTransform;
uniform float U_FogDist;

attribute vec4 VS_Position;
attribute vec4 VS_Normal;
attribute vec4 VS_Tangent;
attribute vec3 VS_Bitan;
attribute vec3 VS_Light1Dir;
attribute vec3 VS_Light2Dir;
attribute vec4 VS_Light1Col;
attribute vec4 VS_Light2Col;
attribute vec4 VS_Colour;
attribute vec2 VS_TexCoord0;
attribute vec2 VS_TexCoord1;
attribute vec4 VS_EnvVisibility;

varying vec4 PS_Colour; // and fog factor
varying vec4 PS_EnvVisibility;
varying vec3 PS_CameraDir;
varying vec3 PS_Light1Dir;
varying vec3 PS_Light1Col;
varying vec3 PS_Light2Dir;
varying vec3 PS_Light2Col;
varying vec2 PS_EnvMapOffset;
varying vec2 PS_TexCoord0;
varying vec2 PS_TexCoord1;

void main(void)
{
    PS_Colour.rgb = VS_Colour.rgb*2.0 - 0.25; // scale should ideally be game-side
    gl_Position = PTransform * MVTransform * VS_Position;
    
    PS_EnvVisibility = VS_EnvVisibility;
    
    vec3 normal_cameraSpace = normalize(vec3(MVTransform * vec4(VS_Normal.xyz, 0.0)));
    vec3 tangent_cameraSpace = normalize(vec3(MVTransform * vec4(VS_Tangent.xyz, 0.0)));
    vec3 bitan_cameraSpace = normalize(vec3(MVTransform * vec4(VS_Bitan, 0.0)));
    mat3 tsprotation = mat3(tangent_cameraSpace.x, bitan_cameraSpace.x, normal_cameraSpace.x,
                            tangent_cameraSpace.y, bitan_cameraSpace.y, normal_cameraSpace.y,
                            tangent_cameraSpace.z, bitan_cameraSpace.z, normal_cameraSpace.z);
    
    vec3 MVPosition = (MVTransform * VS_Position).xyz;
    
    float dist = length(MVPosition);
    
    PS_CameraDir = tsprotation * MVPosition;
    
    PS_Colour.a = clamp(dist/U_FogDist, 0.0, 1.0); // was /9000.0
    
    PS_EnvMapOffset = vec2(VS_Normal.w, VS_Tangent.w);
    PS_EnvMapOffset = ((PS_EnvMapOffset + 1.0)*0.33105) + 0.0244;
    
    PS_Light1Dir = VS_Light1Dir;
    PS_Light1Col = VS_Light1Col.rgb;
    
    PS_Light2Dir = VS_Light2Dir;
    PS_Light2Col = VS_Light2Col.rgb;
    
    PS_TexCoord0 = VS_TexCoord0*2.0;
    PS_TexCoord1 = VS_TexCoord1*2.0;
}
