uniform sampler2D Texture0; // alpha and diffuse map
uniform sampler2D Texture4; // env map

varying lowp vec3 PS_ReflectDir;
varying lowp vec4 PS_Colour;
varying lowp vec3 PS_Light1Dir;
varying lowp vec4 PS_Light1Col;
varying lowp vec3 PS_FresnelCol;
varying mediump vec2 PS_TexCoord0;

void main(void)
{
    mediump vec3 reflectDir = normalize(PS_ReflectDir);
    
    lowp vec4 textureCol = texture2D(Texture0, PS_TexCoord0);
    
    textureCol.rgb = mix(vec3(0.0,0.155,0.46), textureCol.rgb, textureCol.a);
    lowp float specularity = mix(0.3, 0.05, textureCol.a);
    
    mediump float specExponent = specularity;
    specExponent = 1.0 + 100.0*specExponent;
    mediump float specNormalise = (specExponent + 8.0)*0.04; //*0.5;
    
    lowp vec3 light1Col = PS_Light1Col.rgb;
    
    mediump float spec1 = max(dot(reflectDir, PS_Light1Dir), 0.0);
    spec1 = pow(spec1, specExponent)*specNormalise;
    
    lowp vec3 spec = spec1*light1Col;
    
    lowp vec3 texCol = textureCol.rgb;
    
    gl_FragColor.rgb = PS_Colour.rgb * texCol + spec + PS_FresnelCol;
}
