//uniform sampler2D Texture4; // env map

uniform mat4 PTransform;
uniform mat4 MVTransform;

attribute vec4 VS_Position;
attribute vec3 VS_Normal;
attribute vec4 VS_AmbientColour;
attribute vec4 VS_Light1Col;
attribute vec3 VS_Light1Dir;
attribute vec3 VS_Light2Col; // ignored
attribute vec3 VS_Light2Dir; // ignored
attribute vec2 VS_TexCoord0;

varying vec3 PS_ReflectDir;
varying vec3 PS_Light1Dir;
varying vec4 PS_Light1Col;
varying vec3 PS_FresnelCol;
varying vec4 PS_Colour;
varying vec2 PS_TexCoord0;
 
void main(void)
{
    vec3 MVPosition = (MVTransform * VS_Position).xyz;
    gl_Position = PTransform * MVTransform * VS_Position;
    
    vec3 cameraDir = normalize(MVPosition);
    
    float fresnel = dot(cameraDir, VS_Normal);
    PS_ReflectDir = cameraDir - 1.999*fresnel*VS_Normal;
    
    float diff1 = max(dot(VS_Normal, VS_Light1Dir), 0.0);
    vec3 diffuse = diff1*VS_Light1Col.rgb;
    
    fresnel = clamp(1.0 + fresnel, 0.0, 1.0);
    fresnel *= fresnel;
    fresnel *= fresnel;
    lowp vec3 fresnelCol = vec3(fresnel*0.25, fresnel*0.5, fresnel);
    
    PS_Colour = vec4(VS_AmbientColour.rgb + diffuse - 0.15, VS_AmbientColour.a); // should be -0.25, but -0.15 looks better
    
    PS_TexCoord0 = VS_TexCoord0*2.0;
    PS_Light1Col = VS_Light1Col;
    PS_Light1Dir = VS_Light1Dir;
    PS_FresnelCol = fresnelCol;
}
