uniform sampler2D Texture_Diffuse;
uniform sampler2D Texture_Spec_Mask;
uniform sampler2D Texture_Normal;
uniform sampler2D Texture_EnvMap;

uniform lowp vec3 U_Recolour;
uniform lowp vec3 U_AmbientCol;
uniform lowp vec3 U_Light1Col;
uniform lowp vec3 U_Light2Col;
uniform mediump vec3 U_Light1Dir; // world space
uniform mediump vec3 U_Light2Dir; // world space
uniform lowp vec3 U_SkyCol;
uniform lowp vec3 U_GroundCol;
uniform lowp vec3 U_FogCol;
uniform lowp float U_SkyVis;

// todo:
// + try storing sqrt of normal maps so compression dones't cause so many artifacts
// + maybe use uncompressed texture for normal map - results in artifacts

//varying lowp mat3 PS_TangentToWorldSpace; // need this to do normal mapping
varying lowp vec3 PS_TangentToWorldSpace0;
varying lowp vec3 PS_TangentToWorldSpace1;
varying lowp vec3 PS_TangentToWorldSpace2;
varying mediump vec3 PS_CameraDir;
varying lowp float PS_FogFactor;
varying mediump vec2 PS_TexCoord;

void main(void)
{
    //highp vec3 normal = normalize(PS_Normal);
    //highp vec3 normal = vec3(0.0,0.0,1.0);
    highp vec3 normal = texture2D(Texture_Normal, PS_TexCoord).rgb;
    normal.x = normal.x*2.0 - 1.0;
    normal.y = normal.y*2.0 - 1.0;
    normal.x *= abs(normal.x);
    normal.y *= abs(normal.y);
    //normal.z = sqrt(1.0 - normal.x*normal.x - normal.y*normal.y);
    //normal = normalize(PS_TangentToWorldSpace*normal);
    normal = normalize(vec3(dot(normal,PS_TangentToWorldSpace0),dot(normal,PS_TangentToWorldSpace1),dot(normal,PS_TangentToWorldSpace2)));
    //lowp vec3 normal = texture2D(Texture_Normal, PS_TexCoord).rgb;
    mediump vec3 normCamDir = normalize(PS_CameraDir);
    //normal = normalize(normal*1.999 - 1.0); // currently unused
    
    //mediump vec2 diffuseEnvCoords = 0.331 + 0.0244 + normal.xy*0.0244;
    //lowp vec3 diffuseReflection = texture2D(Texture_EnvMap, diffuseEnvCoords).rgb;
    
    lowp vec3 diffuse = texture2D(Texture_Diffuse, PS_TexCoord).rgb;
    lowp vec2 spec_mask = texture2D(Texture_Spec_Mask, PS_TexCoord).rg;
    //diffuse += spec_mask.r*U_Recolour;
    diffuse = max(diffuse, spec_mask.r*U_Recolour);
    //diffuse = mix(diffuse, diffuse*U_Recolour, spec_mask.r);
    
    //mediump float specExponent = spec_mask.g;
    //specExponent = 1.0 + 200.0*specExponent*specExponent;
    //mediump float specNormalise = (specExponent + 8.0)*0.04; //*0.5; //*0.35
    
    mediump float fresnel = dot(normCamDir, normal);
    mediump vec3 reflectDir = normCamDir - 1.999*fresnel*normal;
    
    lowp float specularity = spec_mask.g;
    mediump float specExponent = specularity;
    
    //mediump vec2 texCoord = 0.331 + 0.1655 + reflectDir.xy*0.1655;
    mediump vec2 texCoord = 0.331 + 0.1655 + reflectDir.xy*0.15;
    mediump float texBias = (1.0-specExponent)*5.0;
    lowp vec4 reflectionFull = texture2D(Texture_EnvMap, texCoord, texBias);
    
    //mediump float specularity = spec_mask.g;
    //mediump float specExponent = 150.0;
    //mediump float specNormalise = 6.32*specularity; // (specExponent + 8.0)*0.04*specularity;
    
    //specExponent = 1.0 + 200.0*specExponent*specExponent;
    specExponent = 1.0 + 150.0*specExponent;
    mediump float specNormalise = (specExponent + 8.0)*0.04; //*0.5; //*0.35
    
    // lambert
    lowp float diff1 = max(dot(normal, U_Light1Dir), 0.0);
    lowp float diff2 = max(dot(normal, U_Light2Dir), 0.0);
    
    // phong
    mediump float spec1 = max(dot(reflectDir, U_Light1Dir), 0.0);
    spec1 = pow(spec1, specExponent)*specNormalise;
    mediump float spec2 = max(dot(reflectDir, U_Light2Dir), 0.0);
    spec2 = pow(spec2, specExponent)*specNormalise;
    
    //lowp vec3 colour = ((U_AmbientCol + diff1*U_Light1Col + diff2*U_Light2Col)*diffuse) + (spec1*U_Light1Col + spec2*U_Light2Col);

    fresnel = clamp(1.0 + fresnel, 0.0, 1.0);
    //fresnel = 1.0 + fresnel;
    fresnel *= fresnel;
    fresnel *= fresnel;
    fresnel = 0.15 + (0.85*fresnel*fresnel);
    
    //lowp float diffuseVis = smoothstep(U_SkyVis-0.4, U_SkyVis+0.4, normal.y);
    //lowp float diffuseVis = smoothstep(U_SkyVis-0.1, U_SkyVis+0.7, normal.y);
    lowp float adjSkyVis = U_SkyVis*1.4;
    lowp float diffuseVis = clamp(smoothstep(adjSkyVis-0.7, adjSkyVis+0.2, normal.y), 0.0, 1.0);
    lowp vec3 diffuseReflection = mix(U_GroundCol, U_SkyCol, diffuseVis);
    
    lowp float reflectionGrey = reflectionFull.a;
    //lowp float skyVis = smoothstep(U_SkyVis-0.1, U_SkyVis+0.1, reflectDir.y);
    //lowp float adjSkyVis2 = U_SkyVis*1.1;
    //lowp float skyVis = clamp(smoothstep(adjSkyVis2-0.2, adjSkyVis2-0.1, reflectDir.y), 0.0, 1.0);
    lowp float adjSkyVis2 = U_SkyVis*1.1;
    lowp float blurRange = 0.4*(1.1-specularity);
    //lowp float skyVis = clamp(smoothstep(adjSkyVis2-blurRange, adjSkyVis2+blurRange, reflectDir.y), 0.0, 1.0);
    lowp float skyVis = smoothstep(clamp(adjSkyVis2-blurRange, 0.0, 1.0), clamp(adjSkyVis2+blurRange, 0.0, 1.0), reflectDir.y);
    lowp vec3 reflection = mix((1.999*reflectionGrey)*U_GroundCol, reflectionFull.rgb, skyVis);
    
    lowp vec3 diffuse_col = (U_AmbientCol + diff1*U_Light1Col + diff2*U_Light2Col + diffuseReflection)*diffuse;
    lowp vec3 specular_col = spec1*U_Light1Col + spec2*U_Light2Col + reflection;
    lowp vec3 colour = mix(diffuse_col, specular_col, fresnel);
    
    //colour = reflectDir*0.5 + 0.5;
    //colour = reflection;
    //colour = diffuse_col;
    //colour = 0.45*U_GroundCol;
    //colour = specular_col;
    //colour = normal;
    //colour = normCamDir*0.5 + 0.5;
    //colour = vec3(skyVis, skyVis, skyVis);
    
    gl_FragColor.rgb = mix(colour, U_FogCol, PS_FogFactor);
}
