//uniform sampler2D Texture4; // env map

uniform mat4 PTransform; // camera->screen space
uniform mat4 MVTransform; // object->camera space
uniform mat4 MTransform; // object->world space
uniform vec3 U_CameraPos; // in world space
uniform float U_FogDist;

// in object space
attribute vec4 VS_Position;
attribute vec3 VS_Normal;
attribute vec3 VS_Tangent;
attribute vec3 VS_Bitangent;

attribute vec2 VS_TexCoord;

// opt idea: have non-scaling version of MTransform, use to calc worldNormal etc.

//varying lowp mat3 PS_TangentToWorldSpace; // tegra 3 (and 2?) have driver bug where mat3's can't be passed to fragment shader
varying lowp vec3 PS_TangentToWorldSpace0;
varying lowp vec3 PS_TangentToWorldSpace1;
varying lowp vec3 PS_TangentToWorldSpace2;
varying mediump vec3 PS_CameraDir;
varying lowp float PS_FogFactor;
varying mediump vec2 PS_TexCoord;
 
void main(void)
{
    float dist = length((MVTransform*VS_Position).xyz);
    
    gl_Position = PTransform * MVTransform * VS_Position;
    
    mat3 objToWorld = mat3(MTransform);
    
    /*
    vec3 worldNormal = objToWorld*VS_Normal;
    vec3 worldTangent = objToWorld*VS_Tangent;
    vec3 worldBitangent = objToWorld*VS_Bitangent;
    
    PS_TangentToWorldSpace = mat3(worldTangent.x, worldTangent.y, worldTangent.z,
                                  worldBitangent.x, worldBitangent.y, worldBitangent.z,
                                  worldNormal.x, worldNormal.y, worldNormal.z);
                                  */
    
    vec3 worldNormal = normalize(objToWorld*VS_Normal);
    vec3 worldTangent = normalize(objToWorld*VS_Tangent);
    vec3 worldBitangent = normalize(objToWorld*VS_Bitangent);
                               
    PS_TangentToWorldSpace0 = vec3(worldTangent.x, worldBitangent.x, worldNormal.x);
    PS_TangentToWorldSpace1 = vec3(worldTangent.y, worldBitangent.y, worldNormal.y);
    PS_TangentToWorldSpace2 = vec3(worldTangent.z, worldBitangent.z, worldNormal.z);

    PS_CameraDir = normalize((MTransform*VS_Position).xyz-U_CameraPos);
    
    PS_TexCoord = VS_TexCoord*2.0;
    
    PS_FogFactor = clamp(dist/U_FogDist, 0.0, 1.0); // was /9000.0
}
