uniform sampler2D Texture0; // alpha and diffuse map
uniform sampler2D Texture4; // env map

uniform lowp vec3 U_FogCol;

varying lowp vec3 PS_ReflectDir;
varying lowp vec4 PS_Colour;
varying lowp vec3 PS_Light1Dir;
varying lowp vec4 PS_Light1Col;
varying lowp vec3 PS_Light2Dir;
varying lowp vec3 PS_Light2Col;
//varying lowp vec3 PS_CameraDir;
varying lowp float PS_FogFactor;
varying mediump vec2 PS_TexCoord0;
varying mediump vec2 PS_TexCoord1;

void main(void)
{
    lowp vec3 reflectDir = normalize(PS_ReflectDir);
    
    lowp vec4 textureCol = texture2D(Texture0, PS_TexCoord0);
    
    lowp vec4 reflectionFull = texture2D(Texture4, PS_TexCoord1);
    
    lowp float specularity = PS_Colour.a;
    
    mediump float specExponent = specularity;
    specExponent = 1.0 + 200.0*specExponent*specExponent;
    mediump float specNormalise = (specExponent + 8.0)*0.04; //*0.5; //*0.35
    
    lowp vec3 light1Col = PS_Light1Col.rgb;
    
    lowp float spec1 = max(dot(reflectDir, PS_Light1Dir), 0.0);
    spec1 = pow(spec1, specExponent)*specNormalise;
    lowp float spec2 = max(dot(reflectDir, PS_Light2Dir), 0.0);
    spec2 = pow(spec2, specExponent)*specNormalise;
    
    lowp float reflectionGrey = reflectionFull.a*0.5;
    lowp float skyThreshold = PS_Light1Col.a;
    
    lowp float skyVis = smoothstep(skyThreshold-0.05, skyThreshold+0.05, reflectDir.y);
    lowp vec3 reflection = mix(vec3(reflectionGrey, reflectionGrey, reflectionGrey), reflectionFull.rgb, skyVis);
    
    lowp vec3 spec = spec1*light1Col + spec2*PS_Light2Col + reflection*specularity;
    
    lowp vec3 texCol = textureCol.rgb;
    
    gl_FragColor.a = textureCol.a;
    
    lowp vec3 colour = PS_Colour.rgb * texCol + spec;
    gl_FragColor.rgb = mix(colour, U_FogCol, PS_FogFactor);
}
