uniform sampler2D U_WaterTex;
uniform sampler2D U_EnvTex;

uniform lowp vec3 U_FogCol;

varying mediump vec3 PS_CameraDir;
varying lowp vec3 PS_Light1Dir;
varying lowp vec3 PS_Light1Col;
varying lowp vec3 PS_Light2Dir;
varying lowp vec3 PS_Light2Col;
varying lowp vec4 PS_Colour; // fog factor was separate, but caused problems on n7
//varying lowp float PS_FogFactor;
varying highp vec2 PS_TexCoord0;
varying highp vec2 PS_TexCoord1;

void main(void)
{
    lowp vec3 normCol1 = texture2D(U_WaterTex, PS_TexCoord0).rgb;
    lowp vec3 normCol2 = texture2D(U_WaterTex, PS_TexCoord1).rgb;
    
    lowp vec3 normCol = 0.5*(normCol1+normCol2);
    
    lowp vec3 normal = normalize(normCol*1.999 - 1.0); // renormalise is necessary for decent specular
    //lowp vec3 normal = vec3(0.0,1.0,0.0);
    
    lowp vec3 normCameraDir = normalize(PS_CameraDir);

    //mediump vec2 skyVisMapBase = (floor(PS_TexCoord0*64.0)+0.5)/64.0; // could avoid this by passing in more params, but this is slower
    
    lowp float fresnel = dot(normCameraDir, normal);
    lowp vec3 reflectDir = normCameraDir - 1.999*fresnel*normal;
    
    //highp vec2 skyVisMapOffset = reflectDir.xy;
    //lowp vec4 skyVisMap = texture2D(Texture5, skyVisMapBase + skyVisMapOffset/128.0);
    
    //mediump vec2 envCoords = 0.5 + reflectDir.xy*0.5; // or could do /4.0
    //mediump vec2 envCoords = 0.1655 + reflectDir.xy*0.163;
    mediump vec2 envCoords = 0.4966 + reflectDir.xy*0.163;
    lowp vec4 reflectionFull = texture2D(U_EnvTex, envCoords);
    
    lowp vec3 normLight1Dir = normalize(PS_Light1Dir);
    lowp vec3 normLight2Dir = normalize(PS_Light2Dir);
    
    //lowp vec3 remainingCol = texture2D(Texture0, PS_TexCoord0).rgb;
    //lowp vec2 lightVis = texture2D(Texture3, PS_TexCoord0).ar;
    
    mediump float specExponent = 500.0;
    mediump float specNormalise = (specExponent + 8.0)*0.04; //*0.5; //*0.35
    
    //lowp float light1Vis = lightVis.y;
    //lowp float light2Vis = lightVis.x;
    
    lowp float spec1 = max(dot(reflectDir, normLight1Dir), 0.0);
    spec1 = pow(spec1, specExponent)*specNormalise;
    lowp float spec2 = max(dot(reflectDir, normLight2Dir), 0.0);
    spec2 = pow(spec2, specExponent)*specNormalise;
    
    lowp vec3 specular1 = spec1*PS_Light1Col;
    lowp vec3 specular2 = spec2*PS_Light2Col;
    
    //remainingCol -= 0.25;
    
    // lambert
    lowp float diff1 = normLight1Dir.z; // assumes normal of 0,0,1 and that light is always above face
    lowp float diff2 = normLight2Dir.z; // assumes normal of 0,0,1 and that light is always above face
    lowp vec3 diffuse1 = diff1*PS_Light1Col;
    lowp vec3 diffuse2 = diff2*PS_Light2Col;
    
    /*
    mediump float skyVis = 1.0-reflectDir.y;
    lowp float reflectionGrey = reflectionFull.a*0.2; //*0.5;
    skyVis = clamp((skyVis-0.45)*10.0, 0.0, 1.0);
    lowp vec3 reflection = mix(vec3(reflectionGrey, reflectionGrey, reflectionGrey), reflectionFull.rgb, skyVis);
     */
    lowp vec3 reflection = reflectionFull.rgb;
    
    fresnel = clamp(1.0 + fresnel, 0.0, 1.0);
    fresnel *= fresnel;
    lowp float specularity = 0.05 + ((1.0-0.05)*fresnel*fresnel); // water has 2% reflectance a 90 degrees
    //lowp float specularity = 0.02 + ((1.0-0.02)*fresnel); // water has 2% reflectance a 90 degrees
    
    //lowp float specularity = 0.7;
    
    //lowp vec3 waterColour = vec3(0.04,0.67,0.59);
    //lowp vec3 waterColour = vec3(0.04,0.2,0.4);
    //lowp vec3 waterColour = vec3(0.05,0.44,0.91);
    //lowp vec3 waterColour = vec3(0.03,0.22,0.45);
    //lowp vec3 waterColour = vec3(0.04,0.3,0.45);
    //lowp vec3 waterColour = vec3(0.04,0.2,0.3);
    //lowp vec3 waterColour = vec3(0.2,0.2,0.1);
    //lowp vec3 waterColour = vec3(0.25,0.25,0.1); // good
    //lowp vec3 waterColour = vec3(0.375,0.375,0.15); // brighter, good
    //lowp vec3 waterColour = vec3(0.38,0.42,0.31);
    //lowp vec3 waterColour = vec3(0.38,0.42,0.31)*0.5;
    lowp vec3 waterColour = vec3(0.03,0.225,0.3375);
    
    //gl_FragColor.a = 1.0;
    //lowp vec3 colour = (PS_Colour.rgb+diffuse1+diffuse2)*waterColour*(1.0-specularity) + (specular1+specular2+reflection)*specularity;
    lowp vec3 colour = (PS_Colour.rgb+diffuse1+diffuse2)*waterColour*(1.0-specularity) + (specular1+specular2+reflection)*specularity;
    gl_FragColor.rgb = mix(colour, U_FogCol, PS_Colour.a);
    
    //gl_FragColor.rgb = 0.5*(normal+1.0);
}
