//uniform sampler2D Texture4; // env map
//uniform sampler2D Texture5; // env visibility

uniform mat4 PTransform;
uniform mat4 MVTransform;
uniform float U_Scroll;
uniform float U_FogDist;

attribute vec4 VS_Position;
attribute vec3 VS_Light1Dir;
attribute vec3 VS_Light2Dir;
attribute vec4 VS_Light1Col;
attribute vec4 VS_Light2Col;
attribute vec4 VS_Colour;
attribute vec2 VS_TexCoord0;
attribute vec2 VS_TexCoord1;
attribute vec4 VS_EnvVisibility;

varying vec4 PS_EnvVisibility;
varying vec3 PS_CameraDir;
varying vec3 PS_Light1Dir;
varying vec3 PS_Light1Col;
varying vec3 PS_Light2Dir;
varying vec3 PS_Light2Col;
varying vec4 PS_Colour; // and fog factor
//varying float PS_FogFactor;
varying vec2 PS_TexCoord0;
varying vec2 PS_TexCoord1;
varying vec2 PS_TexCoord2;

void main(void)
{
    PS_TexCoord0 = VS_TexCoord0*2.0;
    
    // this+below could be done on the cpu, and baked into PS_Colour
    //mediump vec2 skyVisMapBase = (floor(PS_TexCoord0*64.0)+0.5)/64.0; // could avoid this by passing in more params, but this is slower
    //lowp vec4 diffuseVisMap = texture2D(Texture5, skyVisMapBase);
    //lowp vec3 diffuseReflection = texture2D(Texture4, vec2(0.0625, 0.0625)).rgb;
    
    gl_Position = PTransform * MVTransform * VS_Position;
    
    vec3 MVPosition = (MVTransform * VS_Position).xyz;
    
    float dist = length(MVPosition);
    
    //PS_CameraDir = MVPosition/dist;
    
    PS_TexCoord1.s = VS_TexCoord1.t*2.0 - U_Scroll;
    PS_TexCoord1.t = VS_TexCoord1.s*2.0 - U_Scroll;
    PS_TexCoord2.s = VS_TexCoord1.s*4.0 + U_Scroll;
    PS_TexCoord2.t = -VS_TexCoord1.t*4.0 - U_Scroll*2.0;
    
    PS_CameraDir = vec3(vec4(MVPosition, 0.0) * MVTransform).xzy;
    PS_CameraDir.x = -PS_CameraDir.x;
    
    // should do swizzling on cpu? (necessary to avoid doing when read from normal map)
    PS_Light1Dir = VS_Light1Dir.xzy;
    PS_Light1Dir.x = -PS_Light1Dir.x;
    PS_Light1Col = VS_Light1Col.rgb;
    
    // should do swizzling on cpu? (necessary to avoid doing when read from normal map)
    PS_Light2Dir = VS_Light2Dir.xzy;
    PS_Light2Dir.x = -PS_Light2Dir.x;
    PS_Light2Col = VS_Light2Col.rgb;
     
    PS_Colour.a = clamp(dist/U_FogDist, 0.0, 1.0); // was /9000.0
    
    PS_EnvVisibility = VS_EnvVisibility;
    
    // this+above could be done on the cpu, and baked into PS_Colour
    //lowp float diffuseVis = dot(diffuseVisMap, VS_EnvVisibility);
    //diffuseReflection = mix(vec3(0.06,0.06,0.06), diffuseReflection, diffuseVis);
    //PS_Colour.rgb = VS_Colour.rgb + diffuseReflection;
    PS_Colour.rgb = VS_Colour.rgb;
}
